/* 
 * This file is part of AndSys 0.4-rc3 distributed under the MIT license.
 * 
 * Copyright (C) 2012, 2013 Giulio Orru`
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package net.sourceforge.andsys;

import android.annotation.SuppressLint;
import android.content.Intent;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.SystemClock;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemSelectedListener;

// TODO: Issue: How to handle screen orientation change when progress dialog and background thread active?
// TODO: Details about Launchable and NOT Launchable App (for User)

public class ActivityApps extends net.sourceforge.andsys.Activity implements OnItemClickListener,
        OnItemSelectedListener {

    //private ActivityApps mActivity = null;
    
    /**
     * 1. Apps List view
     * 
     * to: Maintain the position of applications list when go to another application. and
     * Consistency data on change device orientation
     */
    private ListView mListViewApps = null;

    /**
     * 2. Orderby Spinner
     */
    private Spinner mSpinnerOrderby = null;

    /**
     * 3. Filters Spinner
     */
    private Spinner mSpinnerFilters = null;

    /**
     * 4. Long Click Action in Apps List
     */
    private AdapterView.OnItemLongClickListener mLongClickListener = null;

    /**
     * 5. Progress Dialog
     */
    //31//private ProgressDialog mProgressDialog = null;

    /**
     * 6. Handler (Runnable)
     */
    private Handler mHandler = null;

    /**
     * 7. Runnable per Apps Adapter
     */
    //130902//private Runnable mRunnableOnUiThread = null;

    /**
     * 8. Runnable per Orderby (Progress Dialog): Sort and assign mListViewApps and listAdapter
     */
    private Runnable mRunnableOrderby = null;

    /**
     * Async Task
     */
    LinearLayout mDialogMainList = null;
    RelativeLayout mDialogAsyncTask = null;
    ProgressBar mDialogAsyncTaskBar = null;
    TextView mDialogAsyncTaskPercentage = null;
    ShowDialogAsyncTask showDialogAsyncTask = null;

    /*
     *  By AsyncTask in App list do NOT EXECUTE onClick and onLongClick (Android 1.6, 2.3)
     *  
     *  TODO: DO NOT fix with this attribute!!!!!!!!!!!
     */
    AppAdapter mAppAdapter = null;
    
    /**
     * Count Item
     */
    Button countItem = null;
    
    /**
     * Called when the activity is first created.This is where you should do all of your normal
     * static set up: create views, bind data to lists, etc. This method also provides you with a
     * Bundle containing the activity's previously frozen state, if there was one. Always followed
     * by onStart().
     * 
     * @see android.app.Activity#onCreate(android.os.Bundle)
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // Set the activity content from a layout resource
        setContentView(R.layout.apps__activity);

        
        //mActivity = this;
        
        /*
         * 1. List View useful in Progress Dialog
         */
        mListViewApps = (ListView) findViewById(android.R.id.list);

        /*
         * 2. Orderby Spinner
         */
        mSpinnerOrderby = (Spinner) findViewById(R.id.spinnerPackagesOrderby);
        if (mSpinnerOrderby != null) {

            // Create an ArrayAdapter using the string array and a spinner layout
            // - android.R.layout.simple_spinner_item is the default (align left)
            // - R.layout.spinner_item it's mine (align center)
            ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this,
                    R.array.apps__orderby, R.layout.item__spinner);

            // Specify the layout to use when the list of choices appears
            adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

            // Apply the adapter to the spinner
            mSpinnerOrderby.setAdapter(adapter);
            mSpinnerOrderby.setOnItemSelectedListener(this);
            // fix ERROR: Index out of bound exception
            if (State.orderby < 0 || State.orderby >= Apps.COUNT_ORDERBY) {
                State.orderby = 0;
            }
            mSpinnerOrderby.setSelection(State.orderby);
        }

        /*
         * 3. Filters Spinner
         */
        mSpinnerFilters = (Spinner) findViewById(R.id.spinnerPackagesFilters);
        if (mSpinnerFilters != null) {

            // Create an ArrayAdapter using the string array and a spinner layout
            ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this,
                    R.array.apps__filters, R.layout.item__spinner);

            // Specify the layout to use when the list of choices appears
            adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

            // Apply the adapter to the spinner
            mSpinnerFilters.setAdapter(adapter);
            mSpinnerFilters.setOnItemSelectedListener(this);
            // fix ERROR: Index out of bound exception
            if (State.filters < 0 || State.filters >= Apps.COUNT_FILTERS) {
                State.filters = 0;
            }
            mSpinnerFilters.setSelection(State.filters);
        }

        /*
         * 4. Long Click Action in Apps List
         */
        mLongClickListener = new AdapterView.OnItemLongClickListener() {
            @Override
            public boolean onItemLongClick(AdapterView<?> parent, View v, int position, long id) {
                String packageName = State.apps.get(position).getName();
                String packageFullname = State.apps.get(position).getPackage();

                /*
                 * Launch AndSys?!?!
                 */
                if (packageFullname.equals(getText(R.string.app_package))) {
                    Toast toast = Toast.makeText(getBaseContext(),
                            getText(R.string.apps__launch_hey), Toast.LENGTH_LONG);
                    toast.show();
                } else {

                    /*
                     * Launcher Activity Exists?
                     */
                    if (State.apps.get(position).getFlagLaunchable()) {
                        Toast toast = Toast.makeText(getBaseContext(),
                                getText(R.string.apps__launch) + " \"" + packageName + "\"",
                                Toast.LENGTH_LONG);
                        toast.show();
                        // Launch application
                        Intent intent = getPackageManager().getLaunchIntentForPackage(
                                packageFullname);
                        startActivity(intent);
                        // garbage collection
                        intent = null;

                    } else {
                        Toast toast = Toast.makeText(getBaseContext(),
                                getText(R.string.apps__unable_to_launch) + " \"" + packageName
                                        + "\"", Toast.LENGTH_LONG);
                        toast.show();
                    }
                }
                return true;
            }
        };

        /*
         * 5. Progress Dialog
         */
        //31//mProgressDialog = new ProgressDialog(this);

        /*
         * 6. Handler (Progress Dialog & Runnable)
         */
        mHandler = new Handler();

        /*
         * 7. Runnable per AppAdapter
         */
      //130902//mRunnableOnUiThread = new Runnable() {
      //130902//      @Override
      //130902//      public void run() {
      //130902//
      //130902//          // --- IN BACKGROUND: BEGIN ---
      //130902//
      //130902// 		//   // Acquire Packages List (in the global scope) State
      //130902// 		// Apps.apps(getBaseContext());             
      //130902//         
      //130902//         
      //130902//         
      //130902//         /*
      //130902//           * Update Count Item (Apps List)
      //130902//          */
      //130902//         if (State.apps != null) {
      //130902//              if (State.apps.size() > 0) {
      //130902//                  countItem.setText(State.apps.size()+" "+ getText(R.string.apps__count_item__items));
      //130902//             } else {
      //130902//                 countItem.setText(getText(R.string.apps__count_item__empty_list));
      //130902//             }
      //130902//         } else {
      //130902//             countItem.setText(getText(R.string.apps__count_item__empty_list));
      //130902//         }
      //130902//          
      //130902//         
      //130902//         mSpinnerFilters.setSelection(State.filters);
      //130902//         mSpinnerOrderby.setSelection(State.orderby);
      //130902//         
      //130902//         
      //130902//         
      //130902//         /*
      //130902//           * Initialize list Adapter
      //130902//          * 
      //130902//           * Consistency data on change device orientation
      //130902//          * 
      //130902//           * By AsyncTask in App list do NOT EXECUTE onClick and onLongClick (Android 1.6, 2.3)
      //130902//           */
      //130902//          // The AdapterView does nothing more than manage visibility.
      //130902//          // Adapter: View packages list (from the global scope)
      //130902//          mAppAdapter = new AppAdapter(getBaseContext(), R.layout.list_item, State.apps);
      //130902//          mListViewApps.setAdapter(mAppAdapter);
      //130902//
      //130902//         
      //130902//          // --- IN BACKGROUND: END ---
      //130902//
      //130902//          // Dismiss the progress dialog
      //130902//          //31//mProgressDialog.dismiss();
      //130902//      }

      //130902//  };

        /*
         * 8. Runnable per Orderby (Progress Dialog): Sort and assign mListViewApps and listAdapter
         */
        mRunnableOrderby = new Runnable() {
            @Override
            public void run() {

                // --- IN BACKGROUND: BEGIN ---

                // Order by.. (in the global scope)
                Apps.appsSort();

                /*
                 * Set spinner selection
                 * 
                 * to: fix Apps spinner selection inconsistent when come back from another activity
                 * (apps is null)
                 */
                if (State.filters < 0 || State.filters >= Apps.COUNT_FILTERS) {
                    State.filters = 0;
                }
                if (State.orderby < 0 || State.orderby >= Apps.COUNT_ORDERBY) {
                    State.orderby = 0;
                }
                mSpinnerFilters.setSelection(State.filters);
                mSpinnerOrderby.setSelection(State.orderby);

                /*
                 * Initialize list Adapter
                 * 
                 * Consistency data on change device orientation
                 */
                // The AdapterView does nothing more than manage visibility.
                // Adapter: View packages list (from the global scope)
                AppAdapter appAdapter = new AppAdapter(getBaseContext(), R.layout.item__list,
                        State.apps);
                mListViewApps.setAdapter(appAdapter);

                // --- IN BACKGROUND: END ---

                // Dismiss the progress dialog
                //31//mProgressDialog.dismiss();
            }
        };

        /*
         * Async Task
         */
        mDialogMainList = (LinearLayout) findViewById(R.id.dialogMainList);
        mDialogAsyncTask = (RelativeLayout) findViewById(R.id.dialogAsyncTask);
        mDialogAsyncTaskBar = (ProgressBar) findViewById(R.id.dialogAsyncTaskBar);
        mDialogAsyncTaskPercentage = (TextView) findViewById(R.id.dialogAsyncTaskPercentage);
        showDialogAsyncTask = new ShowDialogAsyncTask();
        {
            // set dialog Visibility
            mDialogAsyncTask.setVisibility(android.view.View.INVISIBLE);
            mDialogMainList.setVisibility(android.view.View.VISIBLE);
        }
        
        /*
         * Count Item
         */
        countItem = (Button) findViewById(R.id.countItem);
        {
            /*
             * Update Count Item (Apps List)
             */
            if (State.apps != null) {
                if (State.apps.size() > 0) {
                    //countItem.setText(State.apps.size()+" "+ getText(R.string.apps__count_item__items));
                    countItem.setText(""+ State.apps.size());
                } else {
                    countItem.setText(getText(R.string.apps__count_item__empty_list));
                }
            } else {
                countItem.setText(getText(R.string.apps__count_item__empty_list));
            }
        }
    }

    @Override
    protected void onStart() {
        super.onStart();

        /*
         * check ready
         */
        if (State.apps == null) {

            /*
             * Load data and assign mListViewApps and listAdapter
             * 
             * // to: Displaying progress bar when waiting
             * mProgressDialog.setTitle(R.string.label__loading);
             * //mProgressDialog.setMessage(getText(R.string.label__loading_text));
             * mProgressDialog.show(); mHandler.postDelayed(mRunnableAppsLoading, 2000);
             * 
             * 
             * // to: Displaying progress bar when waiting
             * mProgressDialog.setTitle(R.string.apps__sorting);
             * //mProgressDialog.setMessage(parent.getItemAtPosition(State.orderby).toString());
             * //mProgressDialog
             * .setMessage(getResources().getStringArray(R.array.apps__orderby)[State.orderby]);
             * mProgressDialog.show(); mHandler.postDelayed(mRunnableOrderby, 2000);
             * 
             * // Restart the Activity with new Data //onStart();
             * 
             * /* AsyncTask: Loading Apps List
             */
            if (showDialogAsyncTask.getStatus() == AsyncTask.Status.FINISHED) {
                showDialogAsyncTask = new ShowDialogAsyncTask();
            }
            if (showDialogAsyncTask.getStatus() == AsyncTask.Status.PENDING) {
                showDialogAsyncTask.execute();
            }

        } else if (State.apps.size() <= 0) {

            /*
             * Load data and assign mListViewApps and listAdapter
             * 
             * // to: Displaying progress bar when waiting
             * mProgressDialog.setTitle(R.string.label__loading);
             * //mProgressDialog.setMessage(getText(R.string.label__loading_text));
             * mProgressDialog.show(); mHandler.postDelayed(mRunnableAppsLoading, 2000);
             * 
             * 
             * // to: Displaying progress bar when waiting
             * mProgressDialog.setTitle(R.string.apps__sorting);
             * //mProgressDialog.setMessage(parent.getItemAtPosition(State.orderby).toString());
             * //mProgressDialog
             * .setMessage(getResources().getStringArray(R.array.apps__orderby)[State.orderby]);
             * mProgressDialog.show(); mHandler.postDelayed(mRunnableOrderby, 2000);
             * 
             * // Restart the Activity with new Data //onStart();
             * 
             * /* AsyncTask: Loading Apps List
             */
            if (showDialogAsyncTask.getStatus() == AsyncTask.Status.FINISHED) {
                showDialogAsyncTask = new ShowDialogAsyncTask();
            }
            if (showDialogAsyncTask.getStatus() == AsyncTask.Status.PENDING) {
                showDialogAsyncTask.execute();
            }

        } else {
            /*
             * READY!
             */
            

            /*
             * Initialize list Adapter
             * 
             * Consistency data on change device orientation
             */
            {

                AppAdapter appAdapter = new AppAdapter(getBaseContext(), R.layout.item__list,
                        State.apps);
                mListViewApps.setAdapter(appAdapter);
            }

            mListViewApps.setClickable(true);
            mListViewApps.setOnItemClickListener(this);
            mListViewApps.setLongClickable(true);

            // Long Click Action
            mListViewApps.setOnItemLongClickListener(mLongClickListener);
            
            
            
            // to: Displaying progress bar when waiting
            //31//mProgressDialog.setTitle(R.string.label__loading);
            //31//mProgressDialog.setMessage(getText(R.string.label__loading_text));
            //31//mProgressDialog.show();
            //mActivity.runOnUiThread(mRunnableOnUiThread);
            //3116//mHandler.postDelayed(mRunnableOnUiThread, 2000);
            
            

        }

    }

    /**
     * Called when the activity will start interacting with the user. At this point your activity is
     * at the top of the activity stack, with user input going to it. Always followed by onPause().
     */
    @Override
    protected void onResume() {
        super.onResume();

        /*
         * check ready
         */
        if (State.apps == null) {

            /*
             * Load data and assign mListViewApps and listAdapter
             * 
             * // to: Displaying progress bar when waiting
             * mProgressDialog.setTitle(R.string.label__loading);
             * //mProgressDialog.setMessage(getText(R.string.label__loading_text));
             * mProgressDialog.show(); mHandler.postDelayed(mRunnableAppsLoading, 2000);
             * 
             * 
             * // to: Displaying progress bar when waiting
             * mProgressDialog.setTitle(R.string.apps__sorting);
             * //mProgressDialog.setMessage(parent.getItemAtPosition(State.orderby).toString());
             * //mProgressDialog
             * .setMessage(getResources().getStringArray(R.array.apps__orderby)[State.orderby]);
             * mProgressDialog.show(); mHandler.postDelayed(mRunnableOrderby, 2000);
             * 
             * // Restart the Activity with new Data //onStart();
             */

            /*
             * AsyncTask: Loading Apps List
             */
            if (showDialogAsyncTask.getStatus() == AsyncTask.Status.FINISHED) {
                showDialogAsyncTask = new ShowDialogAsyncTask();
            }
            if (showDialogAsyncTask.getStatus() == AsyncTask.Status.PENDING) {
                showDialogAsyncTask.execute();
            }

        } else if (State.apps.size() <= 0) {

            /*
             * Load data and assign mListViewApps and listAdapter
             * 
             * // to: Displaying progress bar when waiting
             * mProgressDialog.setTitle(R.string.label__loading);
             * //mProgressDialog.setMessage(getText(R.string.label__loading_text));
             * mProgressDialog.show(); mHandler.postDelayed(mRunnableAppsLoading, 2000);
             * 
             * 
             * // to: Displaying progress bar when waiting
             * mProgressDialog.setTitle(R.string.apps__sorting);
             * //mProgressDialog.setMessage(parent.getItemAtPosition(State.orderby).toString());
             * //mProgressDialog
             * .setMessage(getResources().getStringArray(R.array.apps__orderby)[State.orderby]);
             * mProgressDialog.show(); mHandler.postDelayed(mRunnableOrderby, 2000);
             * 
             * // Restart the Activity with new Data //onStart();
             */

            /*
             * AsyncTask: Loading Apps List
             */
            if (showDialogAsyncTask.getStatus() == AsyncTask.Status.FINISHED) {
                showDialogAsyncTask = new ShowDialogAsyncTask();
            }
            if (showDialogAsyncTask.getStatus() == AsyncTask.Status.PENDING) {
                showDialogAsyncTask.execute();
            }

        } else {

            /*
             * READY!
             */

            
            
            
            // to: Maintain the position of applications list when go to another application.
            mListViewApps.setSelectionFromTop(State.appsPosition, 0);
            
            
            
            
        }
    }

    @Override
    protected void onPause() {
        super.onPause();

        /*
         * check ready
         */
        if (State.apps != null) {
            if (State.apps.size() > 0) {

                /*
                 * READY!
                 */

                // to: Maintain the position of applications list when go to another application.
                int first = mListViewApps.getFirstVisiblePosition();
                // BUT: if select the last app, at come back the last app maybe hide...
                // fixed:
                int last = mListViewApps.getLastVisiblePosition();
                int count = mListViewApps.getCount();

                // count-1 safe last
                if (last < count - 1) {
                    // first position
                    State.appsPosition = first;
                } else {
                    // last position
                    State.appsPosition = last;
                }
            }
        }

    }

    /*
     * Into ListView mListViewApps
     * 
     * @see android.widget.AdapterView.OnItemClickListener#onItemClick(android.widget.AdapterView,
     * android.view.View, int, long)
     */
    @Override
    @SuppressLint("InlinedApi")
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {

        /*
         * check ready
         */
        if (State.apps != null) {
            if (State.apps.size() > 0) {
                /*
                 * READY!
                 */

                String packageName = State.apps.get(position).getName();
                String packageFullname = State.apps.get(position).getPackage();
                Toast toast = Toast.makeText(getBaseContext(), getText(R.string.apps__details_of)
                        + " \"" + packageName + "\"", Toast.LENGTH_LONG);
                toast.show();

                // Launch State details/remmove
                Intent intent = new Intent();
                Uri intentPackageUri = Uri.parse("package:" + packageFullname);
                intent.setData(intentPackageUri);
                String intentPutExtraName;
                if (android.os.Build.VERSION.SDK_INT >= 9) {
                    intent.setAction(android.provider.Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                    intent.addCategory(Intent.CATEGORY_DEFAULT);
                    intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                    intentPutExtraName = new String("package");
                } else {
                    intent.setAction(Intent.ACTION_VIEW);
                    intent.setClassName("com.android.settings",
                            "com.android.settings.InstalledAppDetails");
                    if (android.os.Build.VERSION.SDK_INT == 8) {
                        intentPutExtraName = new String("pkg");
                    } else {
                        intentPutExtraName = new String("com.android.settings.ApplicationPkgName");
                    }
                }
                intent.putExtra(intentPutExtraName, packageFullname);
                startActivity(intent);
                intent.removeExtra(intentPutExtraName);
                // garbage collection
                intent = null;
            }
        }
    }

    /*
     * Into Spinner Orderby and Filters
     * 
     * to: setOnItemSelectedListener(this) OnItemSelectedListener: OnItemSelectedListener
     */
    @Override
    public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {

        /*
         * check ready
         */

        /*
         * READY!
         */

        if (parent == mSpinnerOrderby) {

            /*
             * Spinner Orderby
             */

            // Set Order by.. (in the global scope)
            if (position >= 0 && position < Apps.COUNT_ORDERBY) {
                if (position != State.orderby) {

                    if (State.apps != null) {
                        if (State.apps.size() > 0) {

                            /*
                             * MUST Save Spinner selected option
                             */
                            State.orderby = position;

                            // to: Displaying progress bar when waiting
                            //31//mProgressDialog.setTitle(R.string.apps__sorting);
                            // mProgressDialog.setMessage(parent.getItemAtPosition(State.orderby).toString());
                            // mProgressDialog.setMessage(getResources().getStringArray(R.array.apps__orderby)[State.orderby]);
                            //31//mProgressDialog.show();
                            mHandler.postDelayed(mRunnableOrderby, 1000);
                        }
                    }
                }
            }
        } else {
            // (parent == mSpinnerFilters)

            /*
             * Spinner Filters
             */

            // Set Order by.. (in the global scope)
            if (position >= 0 && position < Apps.COUNT_FILTERS) {
                if (position != State.filters) {

                    // if (State.apps != null) {
                    // if (State.apps.size() > 0) {
                    /*
                     * MUST Save Spinner selected option
                     */
                    State.filters = position;

                    /*
                     * Load data and assign mListViewApps and listAdapter
                     * 
                     * // to: Displaying progress bar when waiting
                     * mProgressDialog.setTitle(R.string.label__loading);
                     * //mProgressDialog.setMessage(getText(R.string.label__loading_text));
                     * mProgressDialog.show(); mHandler.postDelayed(mRunnableAppsLoading, 2000);
                     * 
                     * 
                     * // to: Displaying progress bar when waiting
                     * mProgressDialog.setTitle(R.string.apps__sorting);
                     * //mProgressDialog.setMessage
                     * (parent.getItemAtPosition(State.orderby).toString());
                     * //mProgressDialog.setMessage
                     * (getResources().getStringArray(R.array.apps__orderby)[State.orderby]);
                     * mProgressDialog.show(); mHandler.postDelayed(mRunnableOrderby, 2000);
                     * 
                     * // Restart the Activity with new Data //onStart();
                     */

                    /*
                     * AsyncTask: Loading Apps List
                     */
                    if (showDialogAsyncTask.getStatus() == AsyncTask.Status.FINISHED) {
                        showDialogAsyncTask = new ShowDialogAsyncTask();
                    }
                    if (showDialogAsyncTask.getStatus() == AsyncTask.Status.PENDING) {
                        showDialogAsyncTask.execute();
                    }

                }
            }

        }

    }

    /*
     * OnItemSelectedListener: onNothingSelected
     */
    @Override
    public void onNothingSelected(AdapterView<?> arg0) {
        // TODO Auto-generated method stub
    }

    /**
     * Async Task
     * 
     * Whenever you first start an Android application, a thread called "main" is automatically
     * created. The main thread, also called the UI thread, is very important because it is in
     * charge of dispatching the events to the appropriate widgets and this includes the drawing
     * events. It is also the thread you interact with Android widgets on.
     * 
     * The method doInBackground() executes automatically on a worker thread
     * 
     * onPreExecute(), onPostExecute() and onProgressUpdate() are all invoked on the UI thread
     */
    private class ShowDialogAsyncTask extends AsyncTask<Void, Integer, Void> {

        int progress_status;

        @Override
        protected void onPreExecute() {
            // update the UI immediately after the task is executed
            super.onPreExecute();

            // Toast.makeText(ActivityApps.this, "Invoke onPreExecute()", Toast.LENGTH_LONG).show();

            progress_status = 1;
            mDialogAsyncTaskPercentage.setText(progress_status +"%");
            
            
            // ## begin ## //
            mDialogAsyncTaskBar.setIndeterminate(false);
            mDialogAsyncTask.setVisibility(android.view.View.VISIBLE);
            mDialogMainList.setVisibility(android.view.View.INVISIBLE);

        }

        @Override
        protected Void doInBackground(Void... params) {

            // while(progress_status<100){
            //
            // progress_status += 2;
            //
            // publishProgress(progress_status);
            // SystemClock.sleep(300);
            //
            // }

            /*
             * 1/3. Acquire Packages List (in the global scope) State
             */
            Apps.apps(getBaseContext());
            // ## publish progress ##
            progress_status += 33;
            publishProgress(progress_status);
            SystemClock.sleep(10);

            /*
             * 2/3. Order by.. (in the global scope)
             */
            Apps.appsSort();
            // ## publish progress ##
            progress_status += 33;
            publishProgress(progress_status);
            SystemClock.sleep(10);

            /*
             * 3/3. Set spinner selection
             * 
             * to: fix Apps spinner selection inconsistent when come back from another activity
             * (apps is null)
             */

            if (State.filters < 0 || State.filters >= Apps.COUNT_FILTERS) {
                State.filters = 0;
            }
            if (State.orderby < 0 || State.orderby >= Apps.COUNT_ORDERBY) {
                State.orderby = 0;
            }
            //NOT HERE!// mSpinnerFilters.setSelection(State.filters);
            //NOT HERE!// mSpinnerOrderby.setSelection(State.orderby);
            // ## publish progress ##
            progress_status += 33;
            publishProgress(progress_status);
            SystemClock.sleep(10);

            /*
             * Initialize list Adapter
             * 
             * Consistency data on change device orientation
             */
            // The AdapterView does nothing more than manage visibility.
            // Adapter: View packages list (from the global scope)
            //AppAdapter appAdapter = new AppAdapter(getBaseContext(), R.layout.list_item, State.apps);
            //NOT HERE!// mListViewApps.setAdapter(appAdapter);

            return null;
        }

        @Override
        protected void onProgressUpdate(Integer... values) {
            super.onProgressUpdate(values);

            mDialogAsyncTaskBar.setProgress(values[0]);
            mDialogAsyncTaskPercentage.setText(values[0] + "%");

        }

        @Override
        protected void onPostExecute(Void result) {
            super.onPostExecute(result);

            // Toast.makeText(ActivityApps.this,"Invoke onPostExecute()", Toast.LENGTH_LONG).show();

            mDialogAsyncTaskPercentage.setText("OK");

            // ## ASYNC TASK END ##//
            

        
        
//            /*
//             * Initialize list Adapter
//             * 
//             * Consistency data on change device orientation
//             * 
//             * By AsyncTask in App list do NOT EXECUTE onClick and onLongClick (Android 1.6, 2.3)
//             */
//            // The AdapterView does nothing more than manage visibility.
//            // Adapter: View packages list (from the global scope)
//            mAppAdapter = new AppAdapter(getBaseContext(), R.layout.list_item, State.apps);
//            mListViewApps.setAdapter(mAppAdapter);
            
            // to: Displaying progress bar when waiting
            //mProgressDialog.setTitle(R.string.label__loading);
            //mProgressDialog.setMessage(getText(R.string.label__loading_text));
            //mProgressDialog.show();
            ////////mActivity.runOnUiThread(mRunnableOnUiThread);
        
            
            // ## end ## //
            //mDialogAsyncTaskBar.setIndeterminate(true);
            //mDialogMainList.setVisibility(android.view.View.VISIBLE);
            //mDialogAsyncTask.setVisibility(android.view.View.INVISIBLE);
            
            
            
            // restart this activity
            finish();
            startActivity(getIntent());
        }

    }

    public void onClickButtonRefreshApps(View view) {
    	
        if (showDialogAsyncTask.getStatus() == AsyncTask.Status.FINISHED) {
            showDialogAsyncTask = new ShowDialogAsyncTask();
        }
        if (showDialogAsyncTask.getStatus() == AsyncTask.Status.PENDING) {
            showDialogAsyncTask.execute();
        }
    	
    }
    
}
